# -*- coding: latin-1 -*-

from south.db import db
from django.db import models
from pykeg.core.models import *

class Migration:
    
    def forwards(self, orm):
        
        # Adding model 'DrinkingSessionGroup'
        db.create_table('core_drinkingsessiongroup', (
            ('id', orm['core.DrinkingSessionGroup:id']),
            ('starttime', orm['core.DrinkingSessionGroup:starttime']),
            ('endtime', orm['core.DrinkingSessionGroup:endtime']),
        ))
        db.send_create_signal('core', ['DrinkingSessionGroup'])
        
        # Adding model 'Config'
        db.create_table('core_config', (
            ('id', orm['core.Config:id']),
            ('key', orm['core.Config:key']),
            ('value', orm['core.Config:value']),
        ))
        db.send_create_signal('core', ['Config'])
        
        # Adding model 'BeerType'
        db.create_table('core_beertype', (
            ('id', orm['core.BeerType:id']),
            ('name', orm['core.BeerType:name']),
            ('brewer', orm['core.BeerType:brewer']),
            ('style', orm['core.BeerType:style']),
            ('calories_oz', orm['core.BeerType:calories_oz']),
            ('carbs_oz', orm['core.BeerType:carbs_oz']),
            ('abv', orm['core.BeerType:abv']),
        ))
        db.send_create_signal('core', ['BeerType'])
        
        # Adding model 'RelayLog'
        db.create_table('core_relaylog', (
            ('id', orm['core.RelayLog:id']),
            ('name', orm['core.RelayLog:name']),
            ('status', orm['core.RelayLog:status']),
            ('time', orm['core.RelayLog:time']),
        ))
        db.send_create_signal('core', ['RelayLog'])
        
        # Adding model 'UserPicture'
        db.create_table('core_userpicture', (
            ('id', orm['core.UserPicture:id']),
            ('user', orm['core.UserPicture:user']),
            ('image', orm['core.UserPicture:image']),
            ('active', orm['core.UserPicture:active']),
        ))
        db.send_create_signal('core', ['UserPicture'])
        
        # Adding model 'UserProfile'
        db.create_table('core_userprofile', (
            ('id', orm['core.UserProfile:id']),
            ('user', orm['core.UserProfile:user']),
            ('gender', orm['core.UserProfile:gender']),
            ('weight', orm['core.UserProfile:weight']),
        ))
        db.send_create_signal('core', ['UserProfile'])
        
        # Adding model 'Thermolog'
        db.create_table('core_thermolog', (
            ('id', orm['core.Thermolog:id']),
            ('name', orm['core.Thermolog:name']),
            ('temp', orm['core.Thermolog:temp']),
            ('time', orm['core.Thermolog:time']),
        ))
        db.send_create_signal('core', ['Thermolog'])
        
        # Adding model 'KegTap'
        db.create_table('core_kegtap', (
            ('id', orm['core.KegTap:id']),
            ('name', orm['core.KegTap:name']),
            ('meter_name', orm['core.KegTap:meter_name']),
            ('description', orm['core.KegTap:description']),
            ('current_keg', orm['core.KegTap:current_keg']),
        ))
        db.send_create_signal('core', ['KegTap'])
        
        # Adding model 'Token'
        db.create_table('core_token', (
            ('id', orm['core.Token:id']),
            ('user', orm['core.Token:user']),
            ('keyinfo', orm['core.Token:keyinfo']),
            ('created', orm['core.Token:created']),
        ))
        db.send_create_signal('core', ['Token'])
        
        # Adding model 'KegSize'
        db.create_table('core_kegsize', (
            ('id', orm['core.KegSize:id']),
            ('name', orm['core.KegSize:name']),
            ('volume', orm['core.KegSize:volume']),
        ))
        db.send_create_signal('core', ['KegSize'])
        
        # Adding model 'BAC'
        db.create_table('core_bac', (
            ('id', orm['core.BAC:id']),
            ('user', orm['core.BAC:user']),
            ('drink', orm['core.BAC:drink']),
            ('rectime', orm['core.BAC:rectime']),
            ('bac', orm['core.BAC:bac']),
        ))
        db.send_create_signal('core', ['BAC'])
        
        # Adding model 'Drink'
        db.create_table('core_drink', (
            ('id', orm['core.Drink:id']),
            ('ticks', orm['core.Drink:ticks']),
            ('volume', orm['core.Drink:volume']),
            ('starttime', orm['core.Drink:starttime']),
            ('endtime', orm['core.Drink:endtime']),
            ('user', orm['core.Drink:user']),
            ('keg', orm['core.Drink:keg']),
            ('status', orm['core.Drink:status']),
        ))
        db.send_create_signal('core', ['Drink'])
        
        # Adding model 'UserLabel'
        db.create_table('core_userlabel', (
            ('id', orm['core.UserLabel:id']),
            ('labelname', orm['core.UserLabel:labelname']),
        ))
        db.send_create_signal('core', ['UserLabel'])
        
        # Adding model 'UserDrinkingSession'
        db.create_table('core_userdrinkingsession', (
            ('id', orm['core.UserDrinkingSession:id']),
            ('user', orm['core.UserDrinkingSession:user']),
            ('starttime', orm['core.UserDrinkingSession:starttime']),
            ('endtime', orm['core.UserDrinkingSession:endtime']),
            ('group', orm['core.UserDrinkingSession:group']),
        ))
        db.send_create_signal('core', ['UserDrinkingSession'])
        
        # Adding model 'Brewer'
        db.create_table('core_brewer', (
            ('id', orm['core.Brewer:id']),
            ('name', orm['core.Brewer:name']),
            ('origin_country', orm['core.Brewer:origin_country']),
            ('origin_state', orm['core.Brewer:origin_state']),
            ('origin_city', orm['core.Brewer:origin_city']),
            ('distribution', orm['core.Brewer:distribution']),
            ('url', orm['core.Brewer:url']),
            ('comment', orm['core.Brewer:comment']),
        ))
        db.send_create_signal('core', ['Brewer'])
        
        # Adding model 'Keg'
        db.create_table('core_keg', (
            ('id', orm['core.Keg:id']),
            ('type', orm['core.Keg:type']),
            ('size', orm['core.Keg:size']),
            ('startdate', orm['core.Keg:startdate']),
            ('enddate', orm['core.Keg:enddate']),
            ('channel', orm['core.Keg:channel']),
            ('status', orm['core.Keg:status']),
            ('description', orm['core.Keg:description']),
            ('origcost', orm['core.Keg:origcost']),
        ))
        db.send_create_signal('core', ['Keg'])
        
        # Adding model 'UserDrinkingSessionAssignment'
        db.create_table('core_userdrinkingsessionassignment', (
            ('id', orm['core.UserDrinkingSessionAssignment:id']),
            ('drink', orm['core.UserDrinkingSessionAssignment:drink']),
            ('session', orm['core.UserDrinkingSessionAssignment:session']),
        ))
        db.send_create_signal('core', ['UserDrinkingSessionAssignment'])
        
        # Adding model 'BeerStyle'
        db.create_table('core_beerstyle', (
            ('id', orm['core.BeerStyle:id']),
            ('name', orm['core.BeerStyle:name']),
        ))
        db.send_create_signal('core', ['BeerStyle'])
        
        # Adding ManyToManyField 'UserProfile.labels'
        db.create_table('core_userprofile_labels', (
            ('id', models.AutoField(verbose_name='ID', primary_key=True, auto_created=True)),
            ('userprofile', models.ForeignKey(orm.UserProfile, null=False)),
            ('userlabel', models.ForeignKey(orm.UserLabel, null=False))
        ))
        
    
    
    def backwards(self, orm):
        
        # Deleting model 'DrinkingSessionGroup'
        db.delete_table('core_drinkingsessiongroup')
        
        # Deleting model 'Config'
        db.delete_table('core_config')
        
        # Deleting model 'BeerType'
        db.delete_table('core_beertype')
        
        # Deleting model 'RelayLog'
        db.delete_table('core_relaylog')
        
        # Deleting model 'UserPicture'
        db.delete_table('core_userpicture')
        
        # Deleting model 'UserProfile'
        db.delete_table('core_userprofile')
        
        # Deleting model 'Thermolog'
        db.delete_table('core_thermolog')
        
        # Deleting model 'KegTap'
        db.delete_table('core_kegtap')
        
        # Deleting model 'Token'
        db.delete_table('core_token')
        
        # Deleting model 'KegSize'
        db.delete_table('core_kegsize')
        
        # Deleting model 'BAC'
        db.delete_table('core_bac')
        
        # Deleting model 'Drink'
        db.delete_table('core_drink')
        
        # Deleting model 'UserLabel'
        db.delete_table('core_userlabel')
        
        # Deleting model 'UserDrinkingSession'
        db.delete_table('core_userdrinkingsession')
        
        # Deleting model 'Brewer'
        db.delete_table('core_brewer')
        
        # Deleting model 'Keg'
        db.delete_table('core_keg')
        
        # Deleting model 'UserDrinkingSessionAssignment'
        db.delete_table('core_userdrinkingsessionassignment')
        
        # Deleting model 'BeerStyle'
        db.delete_table('core_beerstyle')
        
        # Dropping ManyToManyField 'UserProfile.labels'
        db.delete_table('core_userprofile_labels')
        
    
    
    models = {
        'auth.group': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '80'}),
            'permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'})
        },
        'auth.permission': {
            'Meta': {'unique_together': "(('content_type', 'codename'),)"},
            'codename': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'content_type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['contenttypes.ContentType']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '50'})
        },
        'auth.user': {
            'date_joined': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'email': ('django.db.models.fields.EmailField', [], {'max_length': '75', 'blank': 'True'}),
            'first_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'groups': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Group']", 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'is_active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'is_staff': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'is_superuser': ('django.db.models.fields.BooleanField', [], {'default': 'False', 'blank': 'True'}),
            'last_login': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'last_name': ('django.db.models.fields.CharField', [], {'max_length': '30', 'blank': 'True'}),
            'password': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'user_permissions': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['auth.Permission']", 'blank': 'True'}),
            'username': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '30'})
        },
        'contenttypes.contenttype': {
            'Meta': {'unique_together': "(('app_label', 'model'),)", 'db_table': "'django_content_type'"},
            'app_label': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'model': ('django.db.models.fields.CharField', [], {'max_length': '100'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '100'})
        },
        'core.bac': {
            'bac': ('django.db.models.fields.FloatField', [], {}),
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Drink']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'rectime': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'core.beerstyle': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'core.beertype': {
            'abv': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'brewer': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Brewer']"}),
            'calories_oz': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'carbs_oz': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'style': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.BeerStyle']"})
        },
        'core.brewer': {
            'comment': ('django.db.models.fields.TextField', [], {'default': "''", 'null': 'True', 'blank': 'True'}),
            'distribution': ('django.db.models.fields.CharField', [], {'default': "'unknown'", 'max_length': '128'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'origin_city': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128', 'null': 'True', 'blank': 'True'}),
            'origin_country': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128'}),
            'origin_state': ('django.db.models.fields.CharField', [], {'default': "''", 'max_length': '128'}),
            'url': ('django.db.models.fields.URLField', [], {'default': "''", 'max_length': '200', 'null': 'True', 'blank': 'True'})
        },
        'core.config': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'key': ('django.db.models.fields.CharField', [], {'unique': 'True', 'max_length': '256'}),
            'value': ('django.db.models.fields.TextField', [], {})
        },
        'core.drink': {
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keg': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Keg']", 'null': 'True', 'blank': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'status': ('django.db.models.fields.CharField', [], {'default': "'valid'", 'max_length': '128'}),
            'ticks': ('django.db.models.fields.PositiveIntegerField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"}),
            'volume': ('django.db.models.fields.PositiveIntegerField', [], {})
        },
        'core.drinkingsessiongroup': {
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.keg': {
            'channel': ('django.db.models.fields.IntegerField', [], {}),
            'description': ('django.db.models.fields.CharField', [], {'max_length': '256'}),
            'enddate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'origcost': ('django.db.models.fields.FloatField', [], {'default': '0'}),
            'size': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.KegSize']"}),
            'startdate': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'type': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.BeerType']"})
        },
        'core.kegsize': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'volume': ('django.db.models.fields.IntegerField', [], {})
        },
        'core.kegtap': {
            'current_keg': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Keg']", 'null': 'True', 'blank': 'True'}),
            'description': ('django.db.models.fields.TextField', [], {'null': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'meter_name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'core.relaylog': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'status': ('django.db.models.fields.CharField', [], {'max_length': '32'}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.thermolog': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '128'}),
            'temp': ('django.db.models.fields.FloatField', [], {}),
            'time': ('django.db.models.fields.DateTimeField', [], {})
        },
        'core.token': {
            'created': ('django.db.models.fields.DateTimeField', [], {'default': 'datetime.datetime.now'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'keyinfo': ('django.db.models.fields.TextField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'core.userdrinkingsession': {
            'endtime': ('django.db.models.fields.DateTimeField', [], {}),
            'group': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.DrinkingSessionGroup']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'starttime': ('django.db.models.fields.DateTimeField', [], {}),
            'user': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['auth.User']"})
        },
        'core.userdrinkingsessionassignment': {
            'drink': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.Drink']"}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'session': ('django.db.models.fields.related.ForeignKey', [], {'to': "orm['core.UserDrinkingSession']"})
        },
        'core.userlabel': {
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'labelname': ('django.db.models.fields.CharField', [], {'max_length': '128'})
        },
        'core.userpicture': {
            'active': ('django.db.models.fields.BooleanField', [], {'default': 'True', 'blank': 'True'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'image': ('django.db.models.fields.files.ImageField', [], {'max_length': '100'}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['auth.User']", 'unique': 'True'})
        },
        'core.userprofile': {
            'gender': ('django.db.models.fields.CharField', [], {'max_length': '8'}),
            'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'labels': ('django.db.models.fields.related.ManyToManyField', [], {'to': "orm['core.UserLabel']"}),
            'user': ('django.db.models.fields.related.OneToOneField', [], {'to': "orm['auth.User']", 'unique': 'True'}),
            'weight': ('django.db.models.fields.FloatField', [], {})
        }
    }
    
    complete_apps = ['core']



# Copyright 2010 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Dump/restore utility methods for Kegbot."""

import sys

from kegbot.util import kbjson

from pykeg.core import models
from pykeg.proto import protolib

def _no_log(msg):
  pass

def dump(output_fp, kbsite, indent=None, log_cb=_no_log):
  """Produce a dump of this Kegbot system to the given filestream.

  In its current format, the dump is plaintext JSON string consisting of all
  important data, including tap configuration, drink history, and user account
  details.

  All "derived" tables are NOT backed up.  These are tables with data that can
  be regenerated at any time without any loss of history.  Specifically:
    - session chunks
    - user session chunks
    - keg session chunks
    - keg stats
    - user stats
    - session stats
    - system events
  """
  res = {}
  items = (
      ('brewers', models.Brewer.objects.all()),
      ('styles', models.BeerStyle.objects.all()),
      ('beertypes', models.BeerType.objects.all()),
      ('thermosensors', kbsite.thermosensors.all().order_by('id')),
      ('kegs', kbsite.kegs.all().order_by('id')),
      ('taps', kbsite.taps.all().order_by('id')),
      ('sessions', kbsite.sessions.all().order_by('id')),
      ('thermologs', kbsite.thermologs.all().order_by('-id')[:60*24]),
      ('users', models.User.objects.all().order_by('id')),
      ('profiles', models.UserProfile.objects.all().order_by('id')),
      ('drinks', kbsite.drinks.all().order_by('id')),
      ('tokens', kbsite.tokens.all().order_by('id')),
  )

  log_cb('Generating backup data ...')
  for name, qs in items:
    log_cb('  .. dumping %s' % name)
    res[name] = list(protolib.ToDict(qs, full=True))

  log_cb('Serializing and writing backup data ...')
  output_fp.write(kbjson.dumps(res, indent=indent))

def restore(input_fp, kbsite, log_cb=_no_log):
  def _log(obj):
    obj_cls = obj.__class__
    obj_name = obj_cls.__name__
    log_cb('  +++ %s: %s' % (obj_name, obj))

  data = kbjson.loads(input_fp.read())

  brewer_map = {}
  for rec in data.get('brewers', []):
    try:
      brewer = models.Brewer.objects.get(id=rec.id)
    except models.Brewer.DoesNotExist:
      brewer = models.Brewer(id=rec.id)
    brewer.name = rec.name
    brewer.country = rec.get('country')
    brewer.origin_state = rec.get('origin_state')
    brewer.origin_city = rec.get('origin_city')
    brewer.production = rec.production
    brewer.url = rec.get('url')
    brewer.description = rec.get('description')
    brewer.save()
    _log(brewer)
    brewer_map[rec.id] = brewer

  style_map = {}
  for rec in data.get('styles', []):
    try:
      style = models.BeerStyle.objects.get(id=rec.id)
    except models.BeerStyle.DoesNotExist:
      style = models.BeerStyle(id=rec.id)
    style.name = rec.name
    style.save()
    _log(style)
    style_map[rec.id] = style

  type_map = {}
  for rec in data.get('beertypes', []):
    try:
      btype = models.BeerType.objects.get(id=rec.id)
    except models.BeerType.DoesNotExist:
      btype = models.BeerType(id=rec.id)
    btype.name = rec.name
    btype.brewer = brewer_map[rec.brewer_id]
    btype.style = style_map[rec.style_id]
    btype.edition = rec.get('edition')
    btype.abv = rec.get('abv')
    btype.calories_oz = rec.get('calories_oz')
    btype.carbs_oz = rec.get('carbs_oz')
    btype.save()
    _log(btype)
    type_map[rec.id] = btype

  kbsite.thermosensors.all().delete()
  for rec in data.get('thermosensors', []):
    sensor = models.ThermoSensor(site=kbsite, id=int(rec.id))
    sensor.raw_name = rec.sensor_name
    sensor.nice_name = rec.nice_name
    sensor.save()
    _log(sensor)

  kbsite.kegs.all().delete()
  for rec in data['kegs']:
    keg = models.Keg(site=kbsite, id=int(rec.id))
    keg.type = type_map[rec.type_id]
    keg.startdate = rec.started_time
    keg.enddate = rec.finished_time
    keg.status = rec.status
    keg.description = rec.get('description')
    keg.spilled_ml = rec.spilled_ml

    size, created = models.KegSize.objects.get_or_create(
        name=rec.size_name, volume_ml=rec.size_volume_ml)
    size.save()
    keg.size = size

    keg.save()
    _log(keg)

  kbsite.taps.all().delete()
  for rec in data['taps']:
    tap = models.KegTap(site=kbsite, id=int(rec.id))
    tap.name = rec.name
    tap.meter_name = rec.meter_name
    tap.ml_per_tick = rec.ml_per_tick
    tap.description = rec.get('description')
    keg_id = int(rec.get('current_keg_id', 0))
    if keg_id:
      tap.current_keg = models.Keg.objects.get(site=kbsite, id=keg_id)
    thermo_id = int(rec.get('thermo_sensor_id', 0))
    if thermo_id:
      tap.temperature_sensor = models.ThermoSensor.objects.get(site=kbsite, id=thermo_id)
    tap.save()
    _log(tap)

  kbsite.sessions.all().delete()
  for rec in data.get('sessions', []):
    session = models.DrinkingSession(site=kbsite, id=int(rec.id))
    session.starttime = rec.start_time
    session.endtime = rec.end_time
    session.volume_ml = rec.volume_ml
    session.name = rec.get('name')
    session.slug = rec.get('slug')
    session.save()
    _log(session)

  for rec in data.get('thermologs', []):
    log = models.Thermolog(site=kbsite, id=int(rec.id))
    log.sensor = models.ThermoSensor.objects.get(site=kbsite, id=int(rec.sensor_id))
    log.temp = rec.temperature_c
    log.time = rec.record_time
    log.save()
    _log(log)

  user_map = {}
  for rec in data.get('users', []):
    user = None

    # If there's already a user registered with this e-mail address, use it.
    if rec.email:
      user_qs = models.User.objects.filter(email=rec.email)
      if user_qs.count():
        user = user_qs[0]
        user_map[rec.username] = user
        _log(user)
        continue

    # Create a new user, creating a new unique username if necessary.
    iter = 0
    username = rec.username
    while True:
      username_qs = models.User.objects.filter(username=username)
      if not username_qs.count():
        break
      iter += 1
      username = '%s_%i' % (rec.username[:30], iter)

    user = models.User(username=username)
    user.first_name = rec.first_name
    user.last_name = rec.last_name
    user.email = rec.email
    user.password = rec.password
    user.is_active = rec.is_active
    user.last_login = rec.last_login
    user.date_joined = rec.date_joined

    # XXX non-prod
    user.is_staff = rec.is_staff
    user.is_superuser = rec.is_superuser

    user.save()
    user_map[rec.username] = user
    _log(user)

  for rec in data.get('profiles', []):
    user = user_map.get(rec.username)
    if not user:
      print 'Warning: profile for non-existant user: %s' % rec.username
      continue
    profile, created = models.UserProfile.objects.get_or_create(user=user)
    profile.save()
    _log(profile)

  kbsite.tokens.all().delete()
  for rec in data.get('tokens', []):
    token = models.AuthenticationToken(site=kbsite, id=int(rec.id))
    token.auth_device = rec.auth_device
    token.token_value = rec.token_value
    username = rec.get('username')
    if username:
      token.user = user_map[username]
    token.enabled = rec.enabled
    token.created = rec.created_time
    token.pin = rec.get('pin')
    token.save()
    _log(token)

  kbsite.drinks.all().delete()
  for rec in data.get('drinks', []):
    drink = models.Drink(site=kbsite, id=int(rec.id))
    drink.ticks = rec.ticks
    drink.volume_ml = rec.volume_ml
    drink.session = models.DrinkingSession.objects.get(site=kbsite, id=int(rec.session_id))
    drink.starttime = rec.pour_time
    drink.duration = rec.get('duration')
    drink.status = rec.status
    keg_id = int(rec.get('keg_id', 0))
    if keg_id:
      drink.keg = models.Keg.objects.get(site=kbsite, id=keg_id)
    username = rec.get('user_id')
    if username:
      drink.user = user_map[username]
    drink.auth_token = rec.get('auth_token')
    drink.save()
    _log(drink)

  log_cb('Regenerating sessions ...')
  _RegenSessions(kbsite)
  log_cb('Regenerating stats ...')
  _RegenStats(kbsite)
  log_cb('Regenerating events ...')
  _RegenEvents(kbsite)

def _RegenSessions(kbsite):
  for d in kbsite.drinks.all().order_by('starttime'):
    d.session.AddDrink(d)

def _RegenStats(kbsite):
  models.SystemStats.objects.filter(site=kbsite).delete()
  models.UserStats.objects.filter(site=kbsite).delete()
  models.KegStats.objects.filter(site=kbsite).delete()
  models.SessionStats.objects.filter(site=kbsite).delete()
  for d in kbsite.drinks.all().order_by('starttime'):
    d._UpdateSystemStats()
    d._UpdateUserStats()
    d._UpdateKegStats()
    d._UpdateSessionStats()

def _RegenEvents(kbsite):
  kbsite.events.all().delete()
  for d in kbsite.drinks.all().order_by('starttime'):
    models.SystemEvent.ProcessDrink(d)



#!/usr/bin/env python
#
# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Kegbot Server setup program.

This program is intended for configuring a fresh install of Kegbot.  It cannot
be used on an active Kegbot install.
"""

from __future__ import absolute_import

import os
import sys
import gflags
import getpass
import pprint
import random
import subprocess

from kegbot.util import app

from pykeg import __version__ as VERSION
from pykeg.core import kb_common

gflags.DEFINE_boolean('use_existing', True,
    'Bootstrap default values from existing local_settings, if available.')

gflags.DEFINE_boolean('interactive', True,
    'Run in interactive mode.')

gflags.DEFINE_boolean('replace_settings', False,
    'Overwrite target settings file, rather than abort, if it already exists.')

gflags.DEFINE_boolean('replace_data', False,
    'Do not abort if data_root already exists.  WARNING: The contents of '
    '<data_root>/static/, if any, will be erased and replaced with static '
    'Kegbot files.')

gflags.DEFINE_string('settings_dir', '~/.kegbot',
    'Settings file directory.  ')

gflags.DEFINE_string('data_root', '~/kegbot-data',
    'Data root for Kegbot.')

gflags.DEFINE_string('db_type', 'sqlite',
    'One of: mysql, sqlite, postgres.')

gflags.DEFINE_string('db_user', 'root',
    'MySQL/Postgres username.  Ignored if using sqlite.')

gflags.DEFINE_string('db_password', '',
    'MySQL/Postgres password.  Ignored if using sqlite.')

gflags.DEFINE_string('db_database', 'kegbot',
    'MySQL/Postgres database name.  Ignored if using sqlite.')

gflags.DEFINE_string('sqlite_filename', 'kegbot.sqlite',
    'File name for the Kegbot sqlite database within `data_root`.  Ignored if not using SQLite.')

gflags.DEFINE_string('use_memcached', True,
    'Configure Kegbot to use memcached. ')

FLAGS = gflags.FLAGS

SETTINGS_TEMPLATE = """# Kegbot local settings.
# Auto-generated by %s version %s.
# Safe to edit by hand. See http://kegbot.org/docs/server/ for more info.

# NEVER set DEBUG to `True` in production.
DEBUG = True
TEMPLATE_DEBUG = DEBUG

""" % (sys.argv[0], VERSION)

# Context values which will be copied to the output settings.
SETTINGS_NAMES = (
  'DATABASES',
  'KEGBOT_ROOT',
  'MEDIA_ROOT',
  'STATIC_ROOT',
  'CACHES',
  'SECRET_KEY',
)

class FatalError(Exception):
  """Cannot proceed."""

def load_existing():
  """Attempts to load the existing local_settings module.

  Returns:
    Loaded module, or None if not loadable.
  """
  try:
    from pykeg.core import importhacks
    existing = __import__('local_settings')
    return existing
  except ImportError:
    return None

def trim(docstring):
  """Docstring trimming function, per PEP 257."""
  if not docstring:
    return ''
  # Convert tabs to spaces (following the normal Python rules)
  # and split into a list of lines:
  lines = docstring.expandtabs().splitlines()
  # Determine minimum indentation (first line doesn't count):
  indent = sys.maxint
  for line in lines[1:]:
    stripped = line.lstrip()
    if stripped:
      indent = min(indent, len(line) - len(stripped))
  # Remove indentation (first line is special):
  trimmed = [lines[0].strip()]
  if indent < sys.maxint:
    for line in lines[1:]:
      trimmed.append(line[indent:].rstrip())
  # Strip off trailing and leading blank lines:
  while trimmed and not trimmed[-1]:
    trimmed.pop()
  while trimmed and not trimmed[0]:
    trimmed.pop(0)
  # Return a single string:
  return '\n'.join(trimmed)

### Setup steps

# These define the actual prompts taken during setup.

class SetupStep(object):
  """A step in Kegbot server configuration.

  The base class has no user interface (flags or prompt); see
  ConfigurationSetupStep for that.
  """
  def get_docs(self):
    """Returns the prompt description text."""
    return trim(self.__doc__)

  def get(self, interactive, ctx):
    if interactive:
      docs = self.get_docs()
      print '-'*80
      print '\n'.join(docs.splitlines()[2:])
      print ''
      print ''

  def validate(self, ctx):
    """Validates user input.

    Args:
      ctx: context
    Raises:
      ValueError: on illegal value
    """
    pass

  def save(self, ctx):
    """Performs the action associated with the step, saving any needed values in
    `ctx`"""
    pass


class ConfigurationSetupStep(SetupStep):
  """A SetupStep that gets and/or applies some configuration value."""
  FLAG = None
  CHOICES = []

  def __init__(self):
    super(ConfigurationSetupStep, self).__init__()
    self.value = None

  def do_prompt(self, prompt, choices=[], default=None):
    """Prompts for and returns a value."""
    choices_text = ''
    if choices:
      choices_text = ' (%s)' % ', '.join(choices)

    default_text = ''
    if default is not None:
      default_text = ' [%s]' % default

    prompt_text = '%s%s%s: ' % (prompt, choices_text, default_text)

    value = raw_input(prompt_text)
    if value == '':
      return default
    return value

  def get_default(self, ctx):
    return self.get_from_flag(ctx)

  def get_from_prompt(self, ctx):
    docs = self.get_docs()
    return self.do_prompt(docs.splitlines()[0], self.CHOICES,
        self.get_default(ctx))

  def get_from_flag(self, ctx):
    if self.FLAG:
      return getattr(FLAGS, self.FLAG)
    return None

  def get(self, interactive, ctx):
    super(ConfigurationSetupStep, self).get(interactive, ctx)
    if interactive:
      ret = self.get_from_prompt(ctx)
    else:
      ret = self.get_from_flag(ctx)
    self.value = ret

  def validate(self, ctx):
    if self.CHOICES and self.value not in self.CHOICES:
      raise ValueError('Value must be one of: %s' % ', '.join(self.CHOICES))
    super(ConfigurationSetupStep, self).validate(ctx)

  def save(self, ctx):
    pass

### Main Steps

class RequiredLibraries(SetupStep):
  def validate(self, ctx):
    try:
        from PIL import Image, ImageColor, ImageChops, ImageEnhance, ImageFile, \
                ImageFilter, ImageDraw, ImageStat
    except ImportError:
        try:
            import Image
            import ImageColor
            import ImageChops
            import ImageEnhance
            import ImageFile
            import ImageFilter
            import ImageDraw
            import ImageStat
        except ImportError:
            raise FatalError('Could not locate Python Imaging Library, '
                'please install it ("pip install pillow" or "apt-get install python-imaging")')


class SettingsDir(ConfigurationSetupStep):
  """Select the settings file location.

  Kegbot's master settings file for this system (local_settings.py) should live
  in one of two places on the filesystem:

    ~/.kegbot/     (local to this user, recommended)
    /etc/kegbot/   (global to all users, requires root access)

  If in doubt, use the default.
  """
  FLAG = 'settings_dir'
  CHOICES = ('~/.kegbot', '/etc/kegbot')

  def validate(self, ctx):
    self.value = os.path.expanduser(self.value)
    if os.path.exists(self.value) and not os.path.isdir(self.value):
      raise ValueError('Settings dir "%s" exists and is a file.' % self.value)
    ctx['SETTINGS_DIR'] = self.value
    if 'SECRET_KEY' not in ctx:
      ctx['SECRET_KEY'] = ''.join([random.choice('abcdefghijklmnopqrstuvwxyz0123456789!@#$%^&*(-_=+)') for i in range(50)])

  def save(self, ctx):
    if not os.path.exists(self.value):
      try:
        os.makedirs(self.value)
      except OSError, e:
        raise FatalError("Couldn't create settings dir '%s': %s" % (self.value, e))


class KegbotDataRoot(ConfigurationSetupStep):
  """Path to Kegbot's data root.

  This should be a directory on your filesystem where Kegbot will create its
  STATIC_ROOT (static files used by the web server, such as css and java script)
  and MEDIA_ROOT (media uploads like user profile pictures).
  """
  FLAG = 'data_root'

  def validate(self, ctx):
    self.value = os.path.expanduser(self.value)
    if os.path.exists(self.value):
      if os.listdir(self.value):
        if not FLAGS.replace_data:
          raise ValueError('Data root "%s" already exists and is not empty.' % self.value)
    media_root = os.path.join(self.value, 'media')
    static_root = os.path.join(self.value, 'static')
    ctx['KEGBOT_ROOT'] = self.value
    ctx['MEDIA_ROOT'] = media_root
    ctx['STATIC_ROOT'] = static_root
    super(KegbotDataRoot, self).validate(ctx)

  def save(self, ctx):
    static_root = ctx['STATIC_ROOT']
    media_root = ctx['MEDIA_ROOT']
    try:
      os.makedirs(static_root)
      os.makedirs(media_root)
    except OSError, e:
      raise FatalError('Could not create directory "%s": %s' % (self.value, e))


class Memcached(ConfigurationSetupStep):
  """Enable memcached caching layer.

  If memcached is installed and available, Kegbot can use it to perform some
  operations faster.  On Ubuntu, you can install memcached this way:
    sudo apt-get install memcached

  It's generally a good idea to enable this.
  """
  FLAG = 'use_memcached'
  HOST = '127.0.0.1:11211'  # todo: make configurable
  CHOICES = ['yes', 'no']

  def get_default(self, ctx):
    return ('no', 'yes')[super(Memcached, self).get_default(ctx)]

  def validate(self, ctx):
    if self.value == 'yes':
      self.value = True
    if self.value == 'no':
      self.value = False
    if self.value not in (True, False):
      raise ValueError('Please enter "yes" or "no".')

    if not self.value:
      return

    client = None
    try:
      import memcache
      client = memcache.Client([self.HOST])
    except ImportError:
      try:
        import pylibmc
        client = pylibmc.Client([self.HOST])
      except ImportError:
        pass

    if not client:
      raise ValueError('Memcache client library is not installed. '
          '("pip install python-memcached" or "pip install pylibmc")')

    if self.value:
      ctx['CACHES'] = {
          'default': {
            'BACKEND':  'django.core.cache.backends.memcached.MemcachedCache',
            'LOCATION': self.HOST,
          }
      }


class ConfigureDatabase(ConfigurationSetupStep):
  """Select database for Kegbot Server backend.

  Currently only MySQL, Postgres, and sqlite are supported by the setup wizard.
  """
  def get_from_flag(self, ctx):
    self.choice = FLAGS.db_type
    if self.choice == 'sqlite':
      return FLAGS.sqlite_filename
    else:
      return (FLAGS.db_user, FLAGS.db_password, FLAGS.db_database)

  def get_from_prompt(self, ctx):
    self.choice = self.do_prompt('Database type',
        choices=('sqlite', 'mysql', 'postgres'), default=FLAGS.db_type)

    if self.choice == 'sqlite':
      root = ctx['KEGBOT_ROOT']
      return self.do_prompt('SQLite database filename to create in %s' % root,
          default=FLAGS.sqlite_filename)
    else:
      user = self.do_prompt('Database user')
      password = getpass.getpass()
      database = self.do_prompt('Database name', default='kegbot')
      return (user, password, database)

  def validate(self, ctx):
    super(ConfigureDatabase, self).validate(ctx)
    if self.choice == 'sqlite':
      root = ctx['KEGBOT_ROOT']
      path = os.path.join(root, self.value)
      if os.path.exists(path):
        raise ValueError('SQLite database file already exists at %s' % path)
    else:
      user, password, database = self.value
      if user == '':
        raise ValueError('Must give a database username')
      elif database == '':
        raise ValueError('Must give a database name')

    if self.choice == 'sqlite':
      cfg = {
        'default': {
          'ENGINE': 'django.db.backends.sqlite3',
          'NAME': os.path.join(ctx['KEGBOT_ROOT'], self.value),
        }
      }
    else:
      user, password, database = self.value
      cfg = {
        'default': {
          'NAME': database,
          'USER': user,
          'PASSWORD': password,
          'HOST': '',
        }
      }
      if self.choice == 'mysql':
        cfg['default']['ENGINE'] = 'django.db.backends.mysql'
        cfg['default']['OPTIONS'] = { 'init_command': 'SET storage_engine=INNODB' }
      else:
        cfg['default']['ENGINE'] = 'django.db.backends.postgresql_psycopg2'

    ctx['DATABASES'] = cfg


STEPS = [
    RequiredLibraries(),
    SettingsDir(),
    Memcached(),
    KegbotDataRoot(),
    ConfigureDatabase(),
]


class SetupApp(app.App):
  def _Setup(self):
    app.App._Setup(self)

  def _SetupSignalHandlers(self):
    pass

  def _MainLoop(self):
    steps = STEPS
    ctx = {}

    if FLAGS.interactive:
      self.build_interactive(ctx)
    else:
      self.build(ctx)

    try:
      self.finish_setup(ctx)
    except (ValueError, FatalError), e:
      print 'ERROR: %s' % e
      sys.exit(1)

  def build(self, ctx):
    for step in STEPS:
      try:
        step.get(interactive=False, ctx=ctx)
        step.validate(ctx)
      except (ValueError, FatalError), e:
        print 'ERROR: %s' % e
        sys.exit(1)

  def build_interactive(self, ctx):
    try:
      import readline
    except ImportError:
      pass

    for step in STEPS:
      while not self._do_quit:
        try:
          step.get(interactive=True, ctx=ctx)
          step.validate(ctx)
          print ''
          print ''
          break
        except KeyboardInterrupt, e:
          print ''
          sys.exit(1)
        except FatalError, e:
          print ''
          print 'ERROR: %s' % e
          sys.exit(1)
        except ValueError, e:
          print ''
          print ''
          print ''
          print 'ERROR: %s' % e


  def finish_setup(self, ctx):
    print ''
    print 'Generated configuration:'
    for key in sorted(SETTINGS_NAMES):
      print '  %s = %s' % (key, repr(ctx.get(key)))
    print ''

    for step in STEPS:
      step.save(ctx)

    settings_file = os.path.join(ctx['SETTINGS_DIR'], 'local_settings.py')
    print 'Writing settings to %s ..' % settings_file

    if os.path.exists(settings_file) and not FLAGS.replace_settings:
      raise ValueError('%s exists and --replace_settings was not given.' % settings_file)

    outfd = open(settings_file, 'w+')
    outfd.write(SETTINGS_TEMPLATE)
    for key in SETTINGS_NAMES:
      if key in ctx:
        outfd.write('%s = %s\n\n' % (key, repr(ctx[key])))
    outfd.close()

    print 'Finishing setup ...'
    existing = load_existing()
    if not existing:
      raise ValueError('Could not import local_settings.')

    existing_file = existing.__file__
    if not existing_file.startswith(settings_file):  # py,pyc
      raise ValueError('Imported settings does not match: imported=%s '
          'expected=%s' % (existing_file, settings_file))

    self.run_command('kegbot syncdb --all --noinput -v 0')
    self.run_command('kegbot migrate --all --fake --noinput -v 0')

    if FLAGS.interactive:
      try:
        self.run_command('kegbot collectstatic --noinput')
      except FatalError, e:
        print 'WARNING: Collecting static files failed: %s' % e
        print ''
        print 'Try again with "kegbot collectstatic"'
    else:
      self.run_command('kegbot collectstatic --noinput')

    print ''
    print 'Done!'
    print ''
    print 'You may now run the built-in web server:'
    print '  $ kegbot runserver'

  def run_command(self, s, allow_fail=False):
    print 'Running command: %s' % s
    ret = subprocess.call(s.split())
    if ret != 0:
      msg = 'Command returned non-zero exit status (%s)' % ret
      if allow_fail:
        print msg
      else:
        raise FatalError(msg)

if __name__ == '__main__':
  SetupApp.BuildAndRun(name='kegbot-setup')



# Copyright 2013 Mike Wakerly <opensource@hoho.com>
#
# This file is part of the Pykeg package of the Kegbot project.
# For more information on Pykeg or Kegbot, see http://kegbot.org/
#
# Pykeg is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 2 of the License, or
# (at your option) any later version.
#
# Pykeg is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Pykeg.  If not, see <http://www.gnu.org/licenses/>.

"""Foursquare plugin for Kegbot."""

from django.conf import settings
from pykeg.plugin import plugin
from pykeg.plugin import util

from kegbot.util import kbjson

import foursquare

from . import forms
from . import tasks
from . import views

KEY_SITE_SETTINGS = 'settings'
KEY_CLIENT_ID = 'client_id'
KEY_CLIENT_SECRET = 'client_secret'
KEY_VENUE_DETAIL = 'venue_detail'

class FoursquarePlugin(plugin.Plugin):
    NAME = 'Foursquare'
    SHORT_NAME = 'foursquare'
    DESCRIPTION = 'Check in when you pour!'
    URL = 'http://kegbot.org'
    VERSION = '0.0.1-pre'

    def get_admin_settings_view(self):
        return views.admin_settings

    def get_user_settings_view(self):
        return views.user_settings

    def get_extra_user_views(self):
        return [
            ('redirect/$', 'pykeg.contrib.foursquare.views.auth_redirect', 'redirect'),
            ('callback/$', 'pykeg.contrib.foursquare.views.auth_callback', 'callback'),
        ]

    def handle_new_event(self, event):
        self.logger.info('Handling new event: %s' % event.id)
        user = event.user

        if event.kind != 'drink_poured':
            self.logger.info('Ignoring event: not session_joined.')
            return

        if not user:
            self.logger.info('Ignoring event: anonymous.')
            return

        if util.is_stale(event.time):
            self.logger.info('Ignoring event: stale.')
            return

        token = self.get_user_token(user)
        if not token:
            self.logger.info('Ignoring event: no token for user %s.' % user.username)
            return

        settings = self.get_user_settings(user)
        if not settings or not settings.get('enable_checkins'):
            self.logger.info('Ignoring event: not enabled.')
            return

        venue_id = self.get_site_settings().get('venue_id')
        if not venue_id:
            self.logger.info('Ignoring event: no venue id.')
            return

        tasks.checkin.delay(token, venue_id)

    ### Foursquare-specific methods

    def get_credentials(self):
        if getattr(settings, 'EMBEDDED', False):
            return (
                getattr(settings, 'FOURSQUARE_CLIENT_ID', ''),
                getattr(settings, 'FOURSQUARE_CLIENT_SECRET', ''),
            )
        data = self.get_site_settings()
        return data.get('client_id'), data.get('client_secret')

    def get_site_settings_form(self):
        form = forms.SiteSettingsForm()
        self.load_form_defaults(form, KEY_SITE_SETTINGS)
        return form

    def get_site_settings(self):
        return self.get_saved_form_data(forms.SiteSettingsForm(), KEY_SITE_SETTINGS)

    def save_site_settings_form(self, form):
        self.save_form(form, KEY_SITE_SETTINGS)

    def get_user_settings_form(self, user):
        form = forms.UserSettingsForm()
        self.load_form_defaults(form, 'user_settings:%s' % user.id)
        return form

    def get_user_settings(self, user):
        return self.get_saved_form_data(forms.UserSettingsForm(), 'user_settings:%s' % user.id)

    def save_user_settings_form(self, user, form):
        self.save_form(form, 'user_settings:%s' % user.id)

    def save_venue_detail(self, detail):
        self.datastore.set(KEY_VENUE_DETAIL, kbjson.dumps(detail))

    def get_venue_detail(self):
        return kbjson.loads(self.datastore.get(KEY_VENUE_DETAIL, 'null'))

    def get_user_profile(self, user):
        return kbjson.loads(self.datastore.get('user_detail:%s' % user.id, 'null'))

    def save_user_profile(self, user, profile):
        self.datastore.set('user_detail:%s' % user.id, kbjson.dumps(profile))

    def get_user_token(self, user):
        return self.datastore.get('user_token:%s' % user.id)

    def save_user_token(self, user, token):
        self.datastore.set('user_token:%s' % user.id, token)
